==> tecfy_core_package-1.0.41.tar.gz: lib/services/values.service.dart <==
part of tecfy_core_client_package;

class ValuesService {
  static dynamic getValue(
      Property property, Map<String, dynamic> data, Entity entity) {
    var valueString = data[property.dbName];
    return valueString;
    //if (valueString == null) return null;

    // ValueTypes type = property.valueType;

    // if (entity.properties
    //     .any((p) => p.dbName == valueString.split('.').first)) {
    //   if (!valueString.contains(".")) {
    //     if (isStringHasValidValue(data[valueString]?.toString(), type))
    //       getValueFromDynamic(data[valueString], type);
    //   } else {
    //     var props = valueString.split('.');
    //     var prop =
    //         entity.properties.where((p) => p.dbName == props.first).first;

    //     if (data[props.first] == null) {
    //       if (data[props.first + 'Id'] != null) {
    //         // get entity by Id and set to data[props.first]

    //       }
    //     }

    //     //if (data[props.first] is Map)
    //     // return getValue(valueString.substring(1), type,data[props.first],null ); // pass the entity
    //     //else
    //     return getValueFromDynamic(data[props.first], type);
    //   }
    // }
    // if (isStringHasValidValue(valueString, type))
    //   return getValueFromString(valueString, type);
    // return null;
  }

  static dynamic getValueByString(
      String valueString, Map<String, dynamic> data, Entity entity) async {
    //valueString = EntityService.lowerFirst(valueString);
    try {
      var valueStrings = valueString
          .split('.')
          .map<String>((e) => EntityService.lowerFirst(e));
      ValueTypes type = ValueTypes.Text;

      var firstProp =
          entity.properties.where((p) => p.dbName == valueStrings.first);
      if (firstProp.length > 0 || valueStrings.length > 1) {
        if (valueStrings.length == 1) {
          type = firstProp.first.valueType;
          var val = getValueFromDynamic(data[firstProp.first.dbName], type);
          return val;
        } else {
          firstProp = entity.properties
              .where((p) => p.dbName == '${valueStrings.first}Id');
          if (firstProp.length == 0)
            print('++++++++++ not property for ${valueStrings.first}Id');
          print(
              '++++++++ trying to ckeck entity ${(firstProp.first as PropertyDropdown).entityName}');
          var secondEntity = EntityService.getEntity(
              (firstProp.first as PropertyDropdown).entityName);

          print('+++++++++ entity is ${secondEntity?.dbName}');
          var secondProp = secondEntity?.properties
                  .where((p) => p.dbName == valueStrings.last) ??
              [];
          if (secondProp.length == 0)
            print('++++++++++ not property for ${valueStrings.last}');
          type = secondProp.first.valueType;
          print(
              '+++++ has sub entity ${secondProp.first.dbName}     type ${secondProp.first.valueType}    value is null ${data[valueStrings.first] == null}');

          if (data[valueStrings.first] == null) {
            if (data[valueStrings.first + 'Id'] != null) {
              // get entity by Id and set to data[valueStrings.first]
              var d = await EntityService.getOne(
                  (firstProp.first as PropertyDropdown).entityName,
                  data[valueStrings.first + 'Id']);
              data[valueStrings.first] = d;
            } else {
              print(
                  '+++++++++ no id selected and entity is null!!  ${valueStrings.first + 'Id'}');
            }
          }

          return getValueFromDynamic(
              //data == null ||
              //    valueStrings == null ||
              valueStrings.length == 0 || data[valueStrings.first] == null
                  ? null
                  : data[valueStrings.first][valueStrings.last],
              type);
        }
      } else
        print('+++++++ cannot find property   ${valueStrings.first}');

      if (isStringHasValidValue(valueString, type))
        return getValueFromString(valueString, type);
    } catch (err, stack) {
      print('XXXXXXXXXXXXXX getValueByString :  $err');
      print(stack);
    }
    return valueString;
  }

  static dynamic getValueFromString(String value, ValueTypes type) {
    switch (type) {
      case ValueTypes.Boolean:
        return (value.toLowerCase() == "true" || value == "1");
      case ValueTypes.Text:
        return value;
      case ValueTypes.Number:
        return int.tryParse(value);
      case ValueTypes.NumberArray:
        var lst = value.split(',').map((e) => int.tryParse(e)).toList();
        lst.remove(null);
        return lst;
      case ValueTypes.TextArray:
        var lst = value.split(',').toList();
        lst.remove(null);
        return lst;
      case ValueTypes.DateTime:
        return BaseModel.parseUTC(value);
      default:
        return value;
    }
  }

  static bool isStringHasValidValue(String? value, ValueTypes type) {
    if (value == null || value.length == 0) return false;
    switch (type) {
      case ValueTypes.Boolean:
        if (["0", "1", "false", "true"].contains(value.toLowerCase()))
          return true;
        break;
      case ValueTypes.Text:
        return true;
      // break;
      case ValueTypes.Number:
        return int.tryParse(value) != null;
      // break;
      case ValueTypes.DateTime:
        if (value.contains('-') || value.contains('\\')) return true;
        break;
      case ValueTypes.NumberArray:
        if (value is List) return true;
        break;
      case ValueTypes.TextArray:
        if (value is List) return true;
        break;
      default:
    }
    return false;
  }

  static dynamic getValueFromDynamic(dynamic value, ValueTypes type) {
    if (value != null && value is String)
      return getValueFromString(value, type);

    switch (type) {
      case ValueTypes.Boolean:
        if (value == null) return false;
        if (value is int) return value == 1;
        if (value as bool) return value;
        return false;
      // break;
      case ValueTypes.Text:
        if (value == null) return '';
        if (value is int) return value.toString();
        return value.toString();
      // break;
      case ValueTypes.Number:
        if (value == null) return 0;
        if (value is int) return value;
        break;
      case ValueTypes.NumberArray:
        if (value == null) return [];
        if (value is List) return value;
        break;
      case ValueTypes.TextArray:
        if (value == null) return [];
        if (value is List) return value;
        break;
      case ValueTypes.DateTime:
        if (value == null) return null;
        if (value is DateTime) return value;
        break;
      default:
    }
    return null;
  }
}

//enum ValueType { String, Int, Bool, Date, IntArray,StringArray }

==> tecfy_core_package-1.0.46.tar.gz: lib/services/values.service.dart <==
part of tecfy_core_client_package;

class ValuesService {
  static dynamic getValue(
      Property property, Map<String, dynamic> data, Entity entity) {
    var valueString = data[property.dbName];
    return valueString;
    //if (valueString == null) return null;

    // ValueTypes type = property.valueType;

    // if (entity.properties
    //     .any((p) => p.dbName == valueString.split('.').first)) {
    //   if (!valueString.contains(".")) {
    //     if (isStringHasValidValue(data[valueString]?.toString(), type))
    //       getValueFromDynamic(data[valueString], type);
    //   } else {
    //     var props = valueString.split('.');
    //     var prop =
    //         entity.properties.where((p) => p.dbName == props.first).first;

    //     if (data[props.first] == null) {
    //       if (data[props.first + 'Id'] != null) {
    //         // get entity by Id and set to data[props.first]

    //       }
    //     }

    //     //if (data[props.first] is Map)
    //     // return getValue(valueString.substring(1), type,data[props.first],null ); // pass the entity
    //     //else
    //     return getValueFromDynamic(data[props.first], type);
    //   }
    // }
    // if (isStringHasValidValue(valueString, type))
    //   return getValueFromString(valueString, type);
    // return null;
  }

  static Future<dynamic> getValueByString(String valueString, Map<String, dynamic> data,
      List<Property> entityProperties) async {
    //valueString = EntityService.lowerFirst(valueString);
    try {
      var valueStrings = valueString
          .split('.')
          .map<String>((e) => EntityService.lowerFirst(e));
      ValueTypes type = ValueTypes.Text;

      var firstProp = entityProperties.where((p) => p.dbName == valueStrings.first);
      if (firstProp.length > 0 || valueStrings.length > 1) {
        if (valueStrings.length == 1) {
          type = firstProp.first.valueType;
          var val = getValueFromDynamic(data[firstProp.first.dbName], type);
          return val;
        } else {
          firstProp =
              entityProperties.where((p) => p.dbName == '${valueStrings.first}Id');
          if (firstProp.length == 0)
            print('++++++++++ not property for ${valueStrings.first}Id');
          print(
              '++++++++ trying to ckeck entity ${(firstProp.first as PropertyDropdown).entityName}');
          var secondEntity = EntityService.getEntity(
              (firstProp.first as PropertyDropdown).entityName);

          print('+++++++++ entity is ${secondEntity?.dbName}');
          var secondProp = secondEntity?.properties
                  .where((p) => p.dbName == valueStrings.last) ??
              [];
          if (secondProp.length == 0)
            print('++++++++++ not property for ${valueStrings.last}');
          type = secondProp.first.valueType;
          print(
              '+++++ has sub entity ${secondProp.first.dbName}     type ${secondProp.first.valueType}    value is null ${data[valueStrings.first] == null}');

          if (data[valueStrings.first] == null) {
            if (data[valueStrings.first + 'Id'] != null) {
              // get entity by Id and set to data[valueStrings.first]
              var d = await EntityService.getOne(
                  (firstProp.first as PropertyDropdown).entityName,
                  data[valueStrings.first + 'Id']);
              data[valueStrings.first] = d;
            } else {
              print(
                  '+++++++++ no id selected and entity is null!!  ${valueStrings.first + 'Id'}');
            }
          }

          return getValueFromDynamic(
              //data == null ||
              //    valueStrings == null ||
              valueStrings.length == 0 || data[valueStrings.first] == null
                  ? null
                  : data[valueStrings.first][valueStrings.last],
              type);
        }
      } else
        print('+++++++ cannot find property   ${valueStrings.first}');

      if (isStringHasValidValue(valueString, type))
        return getValueFromString(valueString, type);
    } catch (err, stack) {
      print('XXXXXXXXXXXXXX getValueByString :  $err');
      print(stack);
    }
    return valueString;
  }


  static dynamic getValueByStringEx(String valueString, Map<String, dynamic> data,
      List<Property> entityProperties)  {
    //valueString = EntityService.lowerFirst(valueString);
    try {
      var valueStrings = valueString
          .split('.')
          .map<String>((e) => EntityService.lowerFirst(e));
      ValueTypes type = ValueTypes.Text;

      var firstProp = entityProperties.where((p) => p.dbName == valueStrings.first);
      if (firstProp.length > 0 || valueStrings.length > 1) {
        if (valueStrings.length == 1) {
          type = firstProp.first.valueType;
          var val = getValueFromDynamic(data[firstProp.first.dbName], type);
          return val;
        } else {
          firstProp =
              entityProperties.where((p) => p.dbName == '${valueStrings.first}Id');
          if (firstProp.length == 0)
            print('++++++++++ not property for ${valueStrings.first}Id');
          print(
              '++++++++ trying to ckeck entity ${(firstProp.first as PropertyDropdown).entityName}');
          var secondEntity = EntityService.getEntity(
              (firstProp.first as PropertyDropdown).entityName);

          print('+++++++++ entity is ${secondEntity?.dbName}');
          var secondProp = secondEntity?.properties
                  .where((p) => p.dbName == valueStrings.last) ??
              [];
          if (secondProp.length == 0)
            print('++++++++++ not property for ${valueStrings.last}');
          type = secondProp.first.valueType;
          print(
              '+++++ has sub entity ${secondProp.first.dbName}     type ${secondProp.first.valueType}    value is null ${data[valueStrings.first] == null}');

          // if (data[valueStrings.first] == null) {
          //   if (data[valueStrings.first + 'Id'] != null) {
          //     // get entity by Id and set to data[valueStrings.first]
          //     var d = await EntityService.getOne(
          //         (firstProp.first as PropertyDropdown).entityName,
          //         data[valueStrings.first + 'Id']);
          //     data[valueStrings.first] = d;
          //   } else {
          //     print(
          //         '+++++++++ no id selected and entity is null!!  ${valueStrings.first + 'Id'}');
          //   }
          // }

          return getValueFromDynamic(
              //data == null ||
              //    valueStrings == null ||
              valueStrings.length == 0 || data[valueStrings.first] == null
                  ? null
                  : data[valueStrings.first][valueStrings.last],
              type);
        }
      } else
        print('+++++++ cannot find property   ${valueStrings.first}');

      if (isStringHasValidValue(valueString, type))
        return getValueFromString(valueString, type);
    } catch (err, stack) {
      print('XXXXXXXXXXXXXX getValueByString :  $err');
      print(stack);
    }
    return valueString;
  }

  static dynamic getValueFromString(String value, ValueTypes type) {
    switch (type) {
      case ValueTypes.Boolean:
        return (value.toLowerCase() == "true" || value == "1");
      case ValueTypes.Text:
        return value;
      case ValueTypes.Number:
        return int.tryParse(value);
      case ValueTypes.NumberArray:
        var lst = value.split(',').map((e) => int.tryParse(e)).toList();
        lst.remove(null);
        return lst;
      case ValueTypes.TextArray:
        var lst = value.split(',').toList();
        lst.remove(null);
        return lst;
      case ValueTypes.DateTime:
        return BaseModel.parseUTC(value);
      default:
        return value;
    }
  }

  static bool isStringHasValidValue(String? value, ValueTypes type) {
    if (value == null || value.length == 0) return false;
    switch (type) {
      case ValueTypes.Boolean:
        if (["0", "1", "false", "true"].contains(value.toLowerCase()))
          return true;
        break;
      case ValueTypes.Text:
        return true;
      // break;
      case ValueTypes.Number:
        return int.tryParse(value) != null;
      // break;
      case ValueTypes.DateTime:
        if (value.contains('-') || value.contains('\\')) return true;
        break;
      case ValueTypes.NumberArray:
        if (value is List) return true;
        break;
      case ValueTypes.TextArray:
        if (value is List) return true;
        break;
      default:
    }
    return false;
  }

  static dynamic getValueFromDynamic(dynamic value, ValueTypes type) {
    if (value != null && value is String)
      return getValueFromString(value, type);

    switch (type) {
      case ValueTypes.Boolean:
        if (value == null) return false;
        if (value is int) return value == 1;
        if (value as bool) return value;
        return false;
      // break;
      case ValueTypes.Text:
        if (value == null) return '';
        if (value is int) return value.toString();
        return value.toString();
      // break;
      case ValueTypes.Number:
        if (value == null) return 0;
        if (value is int) return value;
        break;
      case ValueTypes.NumberArray:
        if (value == null) return [];
        if (value is List) return value;
        break;
      case ValueTypes.TextArray:
        if (value == null) return [];
        if (value is List) return value;
        break;
      case ValueTypes.DateTime:
        if (value == null) return null;
        if (value is DateTime) return value;
        break;
      default:
    }
    return null;
  }
}

//enum ValueType { String, Int, Bool, Date, IntArray,StringArray }
